;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Big Boo Boss, adapted by Davros.
;;
;; Description: A Big Boo Boss from SMW.
;;
;; Uses first extra bit: YES
;; If the extra bit is set, it will set the secret exit. Otherwise it will end the level 
;; normally.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    HIT_POINTS = $03        ; Big Boo Boss hit points

                    EXTRA_BITS = $7FAB10

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
                    PHY
                    JSR SUB_HORZ_POS
                    TYA
                    STA $157C,x
                    PLY
                    RTL                 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"                                    
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR SPRITE_CODE_START   ;  |
                    PLB                     ;  |
                    RTL                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


X_SPEED		    dcb $F0,$10
ACCEL_X		    dcb $FF,$01

Y_SPEED		    dcb $0C,$F4
ACCEL_Y		    dcb $01,$FF

IMAGE_TABLE	    dcb $01,$02,$01,$02


SPRITE_CODE_START   JSR SUB_GFX             ; Big Boo Boss graphics routine
		    JSR SUB_TRANS_PALETTE   ; translucent palette routine

		    LDA $14C8,x             ; \ if the sprite is alive...
		    BNE STATUS              ; / ...don't set the goal

                    LDA EXTRA_BITS,x        ; \ set secret exit if the extra bit is set
                    AND #$04                ;  |
                    BEQ GOAL                ; /
                   
SECRET_EXIT         LDA #$01                ; \ set secret exit
                    STA $141C               ; /

GOAL		    INC $13C6               ; prevent Mario from walking at the level end
		    LDA #$FF                ; \ set goal
		    STA $1493               ; /
		    LDA #$0B                ; \ set ending music
		    STA $1DFB               ; /
                    RTS                     ; return

STATUS		    CMP #$08                ; \ if status != 8, return
		    BNE RETURN              ; /
		    LDA $9D                 ; \ if sprites locked, return
		    BNE RETURN              ; /

		    LDA $C2,x               ; \ if the sprite state is...
		    CMP #$00                ;  | ...zero
		    BEQ MODE0               ; /
		    CMP #$01                ; \ ...one
		    BEQ MODE1               ; /
		    CMP #$02                ; \ ...two
		    BEQ MODE2               ; /
		    CMP #$03                ; \ ...three
		    BEQ MODE3               ; /
		    CMP #$04                ; \ ...four
		    BEQ MODE4               ; /
		    CMP #$05                ; \ ...five
		    BEQ MODE5               ; /
		    CMP #$06                ; \ ...six
		    BEQ MODE6               ; /

MODE0		    JMP CALC_FRAME
MODE1		    JMP APPEAR
MODE2		    JMP MOVEMENT
MODE3		    JMP SET_HIT_POINTS
MODE4		    JMP SET_TIMER
MODE5		    JMP SET_FRAME
MODE6		    JMP DEFEATED
		    RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


CALC_FRAME	    LDA #$03                ; \ set image
		    STA $1602,x             ; /
		    INC $1570,x             ; increment number of frames sprite has been on screen
		    LDA $1570,x             ; \ calculate frame
		    CMP #$90                ;  |
		    BNE RETURN              ; /
		    LDA #$08                ; \ set timer
		    STA $1540,x             ; /
		    INC $C2,x               ; increase sprite state
RETURN		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


APPEAR		    LDA $1540,x             ; \ if the timer is set...
		    BNE RETURN2             ; /
		    LDA #$08                ; \ set timer
		    STA $1540,x             ; /
		    INC $190B               ; increase sprite movement/contact?
		    LDA $190B               ; \ if the sprite movement/contact isn't set...
		    CMP #$02                ;  |
		    BNE NO_SOUND            ; /
		    LDY #$10                ; \ sound effect
		    STY $1DF9               ; /
NO_SOUND	    CMP #$07                ; \ if the sprite movement/contact isn't set...        
		    BNE RETURN2             ; /            
		    INC $C2,x               ; increase sprite state
		    LDA #$40                ; \ set timer
		    STA $1540,x             ; /
RETURN2		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 5
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SET_FRAME	    LDA $1540,x             ; \ if the timer is set...
		    BNE SET_IMAGE           ; /
		    STZ $C2,x               ; restore sprite state
		    LDA #$40                ; \ set frame
		    STA $1570,x             ; /
SET_IMAGE	    LDA #$03                ; \ set image
		    STA $1602,x             ; /
		    BRA SET_TIME


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


MOVEMENT	    STZ $1602,x             ; restore image
		    JSR SPRITE_INTERACT     ; sprite interact routine
SET_TIME	    LDA $15AC,x             ; \ set timer
		    BNE DONT_FACE_MARIO     ; /

		    JSR SUB_HORZ_POS        ; \ if direction has changed since last frame...
		    TYA                     ;  |
		    CMP $157C,x             ;  |
		    BEQ FRAME               ; /
		    LDA #$1F                ; \ ...set time to show turning graphic = 1F
		    STA $15AC,x             ;  |
DONT_FACE_MARIO	    CMP #$10                ;  |
		    BNE DONT_CHANGE_DIR     ; /

		    PHA
		    LDA $157C,x             ; \ flip direction
		    EOR #$01                ;  |
		    STA $157C,x             ; /
		    PLA
DONT_CHANGE_DIR	    LSR A
		    LSR A
		    LSR A
		    TAY
		    LDA IMAGE_TABLE,y       ; \ set image
		    STA $1602,x             ; /

FRAME		    LDA $14                 ; \ calculate frame
		    AND #$07                ;  |
		    BNE FRAME2              ; /

		    LDA $151C,x             ; \ set state
		    AND #$01                ;  |
		    TAY                     ; /
		    LDA $B6,x               ; \ set x speed
		    CLC                     ;  |
		    ADC ACCEL_X,y           ;  | set acceleration
		    STA $B6,x               ;  |
		    CMP X_SPEED,y           ;  | compare x speed
		    BNE FRAME2              ; /
		    INC $151C,x             ; increase state

FRAME2		    LDA $14                 ; \ calculate frame
		    AND #$07                ;  |
		    BNE UPDATE_POSITION     ; /

		    LDA $1528,x             ; \ set state
		    AND #$01                ;  |
		    TAY                     ; /
		    LDA $AA,x               ; \ set y speed
		    CLC                     ;  |
		    ADC ACCEL_Y,y           ;  | set acceleration
		    STA $AA,x               ;  |
		    CMP Y_SPEED,y           ;  | compare y speed
		    BNE UPDATE_POSITION     ; /
		    INC $1528,x             ; increase state

UPDATE_POSITION	    JSL $018022             ; update position based on speed values
		    JSL $01801A             ; update position based on speed values
		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SET_HIT_POINTS	    LDA $1540,x             ; \ if the timer is set...
		    BNE SET_PAL             ; /
		    INC $C2,x               ; increase sprite state
		    LDA #$08                ; \ set timer
		    STA $1540,x             ; /
		    JSL $07F78B             ;
		    INC $1534,x             ; increment sprite hit counter
		    LDA $1534,x             ; \ if hit points are zero...
		    CMP #HIT_POINTS         ;  |
		    BNE RETURN3             ; /
		    LDA #$06                ; \ set sprite state
		    STA $C2,x               ; /
		    JSL $03A6C8             ; disappear every sprite on screen routine
RETURN3		    RTS                     ; return

SET_PAL	            AND #$0E                ; set timer value for the flashing palette
		    EOR $15F6,x             ; set flashing palette
		    STA $15F6,x             ; store sprite palette
		    LDA #$03                ; \ set image
		    STA $1602,x             ; /
		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 4
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SET_TIMER	    LDA $1540,x             ; \ if the timer is set...
		    BNE RETURN4             ; /
		    LDA #$08                ; \ set timer
		    STA $1540,x             ; /
		    DEC $190B               ; decrease sprite movement/contact?
		    BNE RETURN4             ; if movement isn't set, return
		    INC $C2,x               ; increase sprite state
		    LDA #$C0                ; \ set timer
		    STA $1540,x             ; /
RETURN4		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite state 6
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


DEFEATED	    LDA #$02                ; \ status = 2 (being killed by star) 
		    STA $14C8,x             ; /
		    STZ $B6,x               ; no x speed
		    LDA #$D0                ; \ set y speed
		    STA $AA,x               ; /
		    LDA #$23                ; \ sound effect
		    STA $1DF9               ; /
		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; sprite interact routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SPRITE_INTERACT	    LDY #$0B                ; 
LOOP		    LDA $14C8,y             ; \ if the sprite status is..
		    CMP #$09                ;  | ...shell-like
		    BEQ PROCESS_SPRITE      ; /
		    CMP #$0A                ; \ ...throwned shell-like
		    BEQ PROCESS_SPRITE      ; /
NEXT_SPRITE	    DEY                     ;
		    BPL LOOP                ; ...otherwise, loop
		    RTS                     ; return

PROCESS_SPRITE	    PHX                     ; push x
		    TYX                     ; transfer x to y
		    JSL $03B6E5             ; get sprite clipping B routine
		    PLX                     ; pull x
		    JSL $03B69F             ; get sprite clipping A routine
		    JSL $03B72B             ; check for contact routine
		    BCC NEXT_SPRITE         ;

		    LDA #$03                ; \ set sprite state
		    STA $C2,x               ; /
		    LDA #$40                ; \ set timer
		    STA $1540,x             ; /

		    PHX                     ; push x
		    TYX                     ; transfer x to y

		    STZ $14C8,x             ; destroy the sprite

BLOCK_SETUP	    LDA $E4,x               ; \ setup block properties
                    STA $9A                 ;  |
                    LDA $14E0,x             ;  |
                    STA $9B                 ;  |
                    LDA $D8,x               ;  |
                    STA $98                 ;  |
                    LDA $14D4,x             ;  |
                    STA $99                 ; /

EXPLODING_BLOCK	    PHB                     ; \ set the exploding block routine
		    LDA #$02                ;  |
		    PHA                     ;  |
		    PLB                     ;  |
		    LDA #$FF                ;  | $FF = set flashing palette
		    JSL $028663             ;  |
		    PLB                     ; /

		    PLX                     ; pull x
		    LDA #$28                ; \ sound effect
		    STA $1DFC               ; /
		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; transluscent palette routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


TRANS_PALETTE	    dcb $FF,$7F,$63,$0C,$00,$00,$00,$0C,$00,$0C,$00,$0C,$00,$0C,$03,$00
		    dcb $FF,$7F,$E7,$1C,$00,$00,$00,$1C,$00,$1C,$20,$1C,$81,$1C,$07,$00
		    dcb $FF,$7F,$6B,$2D,$00,$00,$00,$2C,$40,$2C,$A2,$2C,$05,$2D,$0B,$00
		    dcb $FF,$7F,$EF,$3D,$00,$00,$60,$3C,$C3,$3C,$26,$3D,$89,$3D,$0F,$00
		    dcb $FF,$7F,$73,$4E,$00,$00,$E4,$4C,$47,$4D,$AA,$4D,$0D,$4E,$13,$10
                    dcb $FF,$7F,$F7,$5E,$00,$00,$68,$5D,$CB,$5D,$2E,$5E,$91,$5E,$17,$20
		    dcb $FF,$7F,$7B,$6F,$00,$00,$EC,$6D,$4F,$6E,$B2,$6E,$15,$6F,$1B,$30
		    dcb $FF,$7F,$FF,$7F,$00,$00,$70,$7E,$D3,$7E,$36,$7F,$99,$7F,$1F,$40


SUB_TRANS_PALETTE   LDY #$24
		    STY $40
		    LDA $190B
		    CMP #$08
		    DEC A
		    BCS SUB_824A
		    LDY #$34
		    STY $40
		    INC A
SUB_824A	    ASL A
		    ASL A
		    ASL A
		    ASL A
		    TAX
		    STZ $00
		    LDY $0681
SET_PALETTE	    LDA TRANS_PALETTE,x
		    STA $0684,y
		    INY
		    INX
		    INC $00
		    LDA $00
		    CMP #$10
		    BNE SET_PALETTE
		    LDX $0681
		    LDA #$10
		    STA $0682,x
		    LDA #$F0
		    STA $0683,x
		    STZ $0694,x
		    TXA
		    CLC
		    ADC #$12
		    STA $0681
		    LDX $15E9
		    RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


TILEMAP             dcb $C0,$E0,$E8,$80,$A0,$A0,$80,$82,$A2,$A2,$82,$84,$A4,$C4,$E4,$86
                    dcb $A6,$C6,$E6,$E8,$C0,$E0,$E8,$80,$A0,$A0,$80,$82,$A2,$A2,$82,$84
		    dcb $A4,$C4,$E4,$86,$A6,$C6,$E6,$E8,$C0,$E0,$E8,$80,$A0,$A0,$80,$82
		    dcb $A2,$A2,$82,$84,$A4,$A4,$84,$86,$A6,$A6,$86,$E8,$E8,$E8,$C2,$E2			
		    dcb $80,$A0,$A0,$80,$82,$A2,$A2,$82,$84,$A4,$C4,$E4,$86,$A6,$C6,$E6			

PROPERTIES	    dcb $00,$00,$40,$00,$00,$80,$80,$00,$00,$80,$80,$00,$00,$00,$00,$00
		    dcb $00,$00,$00,$00,$00,$00,$40,$00,$00,$80,$80,$00,$00,$80,$80,$00
		    dcb $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$40,$00,$00,$80,$80,$00
		    dcb $00,$80,$80,$00,$00,$80,$80,$00,$00,$80,$80,$00,$00,$40,$00,$00			
		    dcb $00,$00,$80,$80,$00,$00,$80,$80,$00,$00,$00,$00,$00,$00,$00,$00			

HORZ_DISP	    dcb $08,$08,$20,$00,$00,$00,$00,$10,$10,$10,$10,$20,$20,$20,$20,$30
		    dcb $30,$30,$30,$FD,$0C,$0C,$27,$00,$00,$00,$00,$10,$10,$10,$10,$1F			
		    dcb $20,$20,$1F,$2E,$2E,$2C,$2C,$FB,$12,$12,$30,$00,$00,$00,$00,$10
		    dcb $10,$10,$10,$1F,$20,$20,$1F,$2E,$2E,$2E,$2E,$F8,$11,$FF,$08,$08
		    dcb $00,$00,$00,$00,$10,$10,$10,$10,$20,$20,$20,$20,$30,$30,$30,$30	
		
VERT_DISP	    dcb $12,$22,$18,$00,$10,$20,$30,$00
		    dcb $10,$20,$30,$00,$10,$20,$30,$00			
		    dcb $10,$20,$30,$18,$16,$16,$12,$22
		    dcb $00,$10,$20,$30,$00,$10,$20,$30
		    dcb $00,$10,$20,$30,$00,$10,$20,$30


SUB_GFX		    JSR GET_DRAW_INFO       ; sets y = OAM offset
		    LDA $1602,x		    ; \ $06 = index to frame start
		    STA $06                 ; /
		    ASL A
		    ASL A
		    STA $03
		    ASL A
		    ASL A
		    ADC $03
		    STA $02
		    LDA $157C,x             ; \ $04 = direction
		    STA $04                 ; / 
		    LDA $15F6,x             ; \ $05 = palette  
		    STA $05                 ; / 

		    LDX #$00                ; loop counter = (number of tiles per frame) - 1
LOOP_START	    PHX                     ; push current tile number

		    LDX $02
		    LDA TILEMAP,x           ; \ store tilemap
		    STA $0302,y             ; /

		    LDA $04                 ; \
		    LSR A                   ;  |
		    LDA PROPERTIES,x        ;  | get tile properties using sprite direction
		    ORA $05                 ;  |
		    BCS NO_FLIP             ;  |
		    EOR #$40                ;  |   ...flip tile
NO_FLIP		    ORA $64                 ;  | put in level properties
		    STA $0303,y             ; / store tile properties

		    LDA HORZ_DISP,x         ; \ tile x position = sprite x location ($00)
		    BCS CLEAR               ;  |
		    EOR #$FF                ;  |
		    INC A                   ;  |
		    CLC                     ;  |
		    ADC #$28                ;  |
CLEAR		    CLC                     ;  |
		    ADC $00                 ;  |
		    STA $0300,y             ; /

		    PLX                     ; pull x
		    PHX                     ; push x

		    LDA $06                 ; \ if the graphics are set...
		    CMP #$03                ;  |
		    BCC SET_VERT_DISP       ; /
		    TXA
		    CLC
		    ADC #$14
		    TAX

SET_VERT_DISP	    LDA $01                 ; \ tile y position = sprite y location ($01)
                    CLC                     ;  |
                    ADC VERT_DISP,x         ;  |
                    STA $0301,y             ; /

                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
		    INC $02                 ;  |
		    INX                     ;  |
		    CPX #$14                ;  |
		    BNE LOOP_START          ; / 

		    LDX $15E9
		    LDA $1602,x             ; \ if the image is set...
		    CMP #$03                ;  |
		    BNE DRAW_TILES          ; /

		    LDA $1558,x             ; \ if the timer is set...
		    BEQ DRAW_TILES          ; /

		    LDY $15EA,x
		    LDA $0301,y             ; \ tile y position = sprite y location ($01)
		    CLC                     ;  |
		    ADC #$05                ;  |
		    STA $0301,y             ;  |
		    STA $0305,y             ; /

DRAW_TILES	    LDY #$02                ; \ 02, because we didn't write to 460 yet
                    LDA #$13                ;  | A = number of tiles drawn - 1
		    JSL $01B7B3             ; / don't draw if offscreen
		    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

